@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 
 # This file is part of IVRE.
-# Copyright 2011 - 2022 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2024 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -25,30 +25,25 @@ active (nmap & view) purposes.
 import json
 import os
 import re
-from bisect import bisect_left
-from collections import Counter
 from datetime import datetime
 from itertools import chain
 from textwrap import wrap
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Generator,
-    Iterable,
-    List,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-    cast,
-)
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union, cast
 from urllib.parse import urlparse
 
 from ivre.active.cpe import add_cpe_values
-from ivre.config import DATA_PATH, VIEW_SYNACK_HONEYPOT_COUNT
-from ivre.data.abuse_ch.sslbl import SSLBL_CERTIFICATES, SSLBL_JA3
+from ivre.active.nmap import ALIASES_TABLE_ELEMS
+from ivre.config import VIEW_MAX_HOSTNAMES_COUNT, VIEW_SYNACK_HONEYPOT_COUNT
 from ivre.data.microsoft.exchange import EXCHANGE_BUILDS
+from ivre.plugins import load_plugins
+from ivre.tags import TAG_CDN, TAG_HONEYPOT, add_tags
+from ivre.tags.active import (
+    has_toomany_hostnames,
+    is_real_service_port,
+    is_synack_honeypot,
+    set_auto_tags,
+    set_openports_attribute,
+)
 from ivre.types import NmapServiceMatch, ParsedCertificate, Tag
 from ivre.types.active import (
     HttpHeader,
@@ -61,124 +56,15 @@ from ivre.types.active import (
 from ivre.utils import (
     IPV4ADDR,
     LOGGER,
-    TORCERT_SUBJECT,
     decode_b64,
     encode_b64,
-    get_addr_type,
     get_cert_info,
     get_domains,
-    ip2int,
     key_sort_dom,
-    make_range_tables,
-    net2range,
     nmap_encode_data,
     ports2nmapspec,
 )
 
-ALIASES_TABLE_ELEMS = {
-    # Use the same structured output for both ssl-cert and ssl-cacert
-    "ssl-cacert": "ssl-cert",
-    # Use the same structured output for all the Nuclei scripts
-    "http-nuclei": "nuclei",
-    "network-nuclei": "nuclei",
-    # ls unified output (ls NSE module + ftp-anon)
-    #   grep -lF 'ls.new_vol' * | sed 's#^#    "#;s#.nse$#": "ls",#'
-    "afp-ls": "ls",
-    "http-ls": "ls",
-    "nfs-ls": "ls",
-    "smb-ls": "ls",
-    #   + ftp-anon
-    "ftp-anon": "ls",
-    # vulns unified output (vulns NSE module)
-    #   grep -l -F vulns.Report * | sed 's#^#    "#;s#.nse$#": "vulns",#'
-    "afp-path-vuln": "vulns",
-    "clamav-exec": "vulns",
-    "distcc-cve2004-2687": "vulns",
-    "ftp-libopie": "vulns",
-    "ftp-vsftpd-backdoor": "vulns",
-    "ftp-vuln-cve2010-4221": "vulns",
-    "http-avaya-ipoffice-users": "vulns",
-    "http-cross-domain-policy": "vulns",
-    "http-dlink-backdoor": "vulns",
-    "http-frontpage-login": "vulns",
-    "http-huawei-hg5xx-vuln": "vulns",
-    "http-iis-short-name-brute": "vulns",
-    "http-method-tamper": "vulns",
-    "http-phpmyadmin-dir-traversal": "vulns",
-    "http-phpself-xss": "vulns",
-    "http-sap-netweaver-leak": "vulns",
-    "http-shellshock": "vulns",
-    "http-slowloris-check": "vulns",
-    "http-tplink-dir-traversal": "vulns",
-    "http-vuln-cve2006-3392": "vulns",
-    "http-vuln-cve2009-3960": "vulns",
-    "http-vuln-cve2010-2861": "vulns",
-    "http-vuln-cve2011-3192": "vulns",
-    "http-vuln-cve2011-3368": "vulns",
-    "http-vuln-cve2012-1823": "vulns",
-    "http-vuln-cve2013-0156": "vulns",
-    "http-vuln-cve2013-6786": "vulns",
-    "http-vuln-cve2013-7091": "vulns",
-    "http-vuln-cve2014-2126": "vulns",
-    "http-vuln-cve2014-2127": "vulns",
-    "http-vuln-cve2014-2128": "vulns",
-    "http-vuln-cve2014-2129": "vulns",
-    "http-vuln-cve2014-3704": "vulns",
-    "http-vuln-cve2014-8877": "vulns",
-    "http-vuln-cve2015-1427": "vulns",
-    "http-vuln-cve2015-1635": "vulns",
-    "http-vuln-cve2017-1001000": "vulns",
-    "http-vuln-cve2017-5638": "vulns",
-    "http-vuln-cve2017-5689": "vulns",
-    "http-vuln-cve2017-8917": "vulns",
-    "http-vuln-misfortune-cookie": "vulns",
-    "http-vuln-wnr1000-creds": "vulns",
-    "ipmi-cipher-zero": "vulns",
-    "mysql-vuln-cve2012-2122": "vulns",
-    "qconn-exec": "vulns",
-    "rdp-vuln-ms12-020": "vulns",
-    "realvnc-auth-bypass": "vulns",
-    "rmi-vuln-classloader": "vulns",
-    "rsa-vuln-roca": "vulns",
-    "samba-vuln-cve-2012-1182": "vulns",
-    "smb-double-pulsar-backdoor": "vulns",
-    "smb-vuln-conficker": "vulns",
-    "smb-vuln-cve-2017-7494": "vulns",
-    "smb-vuln-cve2009-3103": "vulns",
-    "smb-vuln-ms06-025": "vulns",
-    "smb-vuln-ms07-029": "vulns",
-    "smb-vuln-ms08-067": "vulns",
-    "smb-vuln-ms10-054": "vulns",
-    "smb-vuln-ms10-061": "vulns",
-    "smb-vuln-ms17-010": "vulns",
-    "smb-vuln-regsvc-dos": "vulns",
-    "smb-vuln-webexec": "vulns",
-    "smb2-vuln-uptime": "vulns",
-    "smtp-vuln-cve2011-1720": "vulns",
-    "smtp-vuln-cve2011-1764": "vulns",
-    "ssl-ccs-injection": "vulns",
-    "ssl-dh-params": "vulns",
-    "ssl-heartbleed": "vulns",
-    "ssl-poodle": "vulns",
-    "sslv2-drown": "vulns",
-    "supermicro-ipmi-conf": "vulns",
-    "tls-ticketbleed": "vulns",
-    # ntlm unified output (*-ntlm-info modules)
-    #   ls *ntlm* | sed 's#^#    "#;s#.nse$#": "ntlm-info",#'
-    "http-ntlm-info": "ntlm-info",
-    "imap-ntlm-info": "ntlm-info",
-    "ms-sql-ntlm-info": "ntlm-info",
-    "nntp-ntlm-info": "ntlm-info",
-    "pop3-ntlm-info": "ntlm-info",
-    "rdp-ntlm-info": "ntlm-info",
-    "smtp-ntlm-info": "ntlm-info",
-    "telnet-ntlm-info": "ntlm-info",
-}
-
-
-BIG_IP_ERROR_BANNER = re.compile("^BIG-IP: \\[0x[0-9a-f]{7}:[0-9]{1,5}\\] ")
-SONICWALL_ERROR_BANNER = re.compile("^\\(Ref.Id: \\?.*\\?\\)$")
-
 
 def create_ssl_output(info: ParsedCertificate) -> List[str]:
     out = []
@@ -269,568 +155,6 @@ def add_cert_hostnames(cert: ParsedCertificate, hostnames: List[NmapHostname]) -
             add_hostname(netloc, "cert-san-uri", hostnames)
 
 
-def is_real_service_port(port: NmapPort) -> bool:
-    """Decides whether a port has a "real" service (=> True) or if it is
-    **possibly** a SYN-ACK "honeypot" (=> False).
-
-    The idea is that a port is "real" if it has a reason that is not
-    "syn-ack" (or similar), or if it has a service_name, or if it has a
-    least one script.
-
-    Host scripts (port == -1) are also considered True.
-
-    """
-    if port.get("port") == -1:
-        return True
-    if port.get("state_reason") and not (
-        # might be "syn-ack" but might as
-        # well be "syn-ack-cwr" or
-        # "syn-psh-ack"
-        port["state_reason"].startswith("syn-")
-        or port["state_reason"] == "passive"
-    ):
-        return True
-    if port.get("service_name") and port["service_name"] != "tcpwrapped":
-        return True
-    if port.get("scripts"):
-        # Ports with scripts usually are "real" service ports, **but**
-        # when a port only has a banner script, the output of which
-        # matches a known SYN-ACK responder answser, we consider it is
-        # **possibly** a SYN-ACK "honeypot" (or responder) and return
-        # False
-        if len(port["scripts"]) == 1 and port["scripts"][0]["id"] == "banner":
-            banner = port["scripts"][0]["output"]
-            if banner == "\n":
-                return False
-            if BIG_IP_ERROR_BANNER.search(banner):
-                return False
-            if SONICWALL_ERROR_BANNER.search(banner):
-                return False
-        return True
-    return False
-
-
-def set_openports_attribute(host: NmapHost) -> None:
-    """This function sets the "openports" value in the `host` record,
-    based on the elements of the "ports" list. This is used in MongoDB to
-    speed up queries based on open ports.
-
-    """
-    openports = host["openports"] = {"count": 0}
-    for port in host.get("ports", []):
-        if port.get("state_state") != "open":
-            continue
-        cur = openports.setdefault(port["protocol"], {"count": 0, "ports": []})
-        if port["port"] not in cur["ports"]:
-            openports["count"] += 1
-            cur["count"] += 1
-            cur["ports"].append(port["port"])
-
-
-def _prepare_tag(tag: Dict[str, Any]) -> Dict[str, Any]:
-    """This function uses a set() for the "info" value, while a list() is
-    used to store it. It is used in add_tags().
-
-    """
-    if "info" in tag:
-        tag["info"] = set(tag["info"])
-    return tag
-
-
-def _clean_tag(tag: Dict[str, Any]) -> Dict[str, Any]:
-    """This function is the opposite of `_prepare_tag()`. It is used in
-    add_tags().
-
-    """
-    if "info" in tag:
-        tag["info"] = sorted(tag["info"])
-    return tag
-
-
-def add_tags(host: NmapHost, tags: Iterable[Tag]) -> None:
-    """This function sets or update the "tags" attribute in `host` by
-    adding or updating the provided `tags`.
-
-    """
-    cur_tags = {tag["value"]: _prepare_tag(tag) for tag in host.get("tags", [])}
-    for tag in tags:
-        cur_tag = cur_tags.setdefault(
-            tag["value"], {"value": tag["value"], "type": tag["type"]}
-        )
-        if "info" in tag:
-            cur_tag.setdefault("info", set()).update(tag["info"])
-    if cur_tags:
-        host["tags"] = [_clean_tag(cur_tags[key]) for key in sorted(cur_tags)]
-
-
-def is_synack_honeypot(host: NmapHost) -> bool:
-    """Returns True iff the host has the "Honeypot" tag with "SYN+ACK
-    honeypot" info.
-
-    """
-    return any(
-        tag["value"] == "Honeypot" and "SYN+ACK honeypot" in tag.get("info", [])
-        for tag in host.get("tags", [])
-    )
-
-
-TAG_CDN: Tag = {"value": "CDN", "type": "info"}
-TAG_DEFAULT_PASSWORD: Tag = {"value": "Default password", "type": "danger"}
-TAG_HONEYPOT: Tag = {"value": "Honeypot", "type": "warning"}
-TAG_MALWARE: Tag = {"value": "Malware", "type": "danger"}
-TAG_RESERVED: Tag = {"value": "Reserved address", "type": "info"}
-TAG_SCANNER: Tag = {"value": "Scanner", "type": "warning"}
-TAG_TOR: Tag = {"value": "TOR", "type": "info"}
-TAG_VULN: Tag = {"value": "Vulnerable", "type": "danger"}
-TAG_VULN_LIKELY: Tag = {"value": "Likely vulnerable", "type": "warning"}
-TAG_VULN_CANNOT_TEST: Tag = {"value": "Cannot test vuln", "type": "info"}
-
-
-_SERVICE_FIELDS = [
-    "service_name",
-    "service_product",
-    "service_version",
-    "service_extrainfo",
-]
-_TOR_SERVICES = {
-    "tor",
-    "tor-control",
-    "tor-info",
-    "tor-orport",
-    "tor-socks",
-}
-_TOR_HTTP_PRODUCTS = {
-    "Tor directory",
-    "Tor directory server",
-    "Tor built-in httpd",
-}
-
-
-_TOR_NODES: Optional[Set[str]] = None
-_CDN_TABLE: Optional[Tuple[List[int], List[Optional[str]]]] = None
-_SCANNERS_TABLE: Optional[Tuple[List[int], List[Optional[str]]]] = None
-
-
-def _get_data() -> None:
-    global _TOR_NODES, _SCANNERS_TABLE, _CDN_TABLE
-    assert DATA_PATH is not None
-    if _TOR_NODES is None:
-        try:
-            with open(
-                os.path.join(DATA_PATH, "tor_exit_nodes.txt"), encoding="utf8"
-            ) as fdesc:
-                _TOR_NODES = {line.strip() for line in fdesc}
-        except FileNotFoundError:
-            LOGGER.warning(
-                "Cannot find file [tor_exit_nodes.txt]. Try running `ivre getwebdata`"
-            )
-            _TOR_NODES = set()
-    if _SCANNERS_TABLE is None:
-        ranges: List[Tuple[str, str, str]] = []
-        try:
-            with open(
-                os.path.join(DATA_PATH, "ssigouvfr_scanners.txt"), encoding="utf8"
-            ) as fdesc:
-                ranges.extend(
-                    (addr, addr, "ANSSI") for addr in (line.strip() for line in fdesc)
-                )
-        except FileNotFoundError:
-            LOGGER.warning(
-                "Cannot find file [ssigouvfr_scanners.txt]. Try running `ivre getwebdata`"
-            )
-        try:
-            with open(
-                os.path.join(DATA_PATH, "ukncsc_scanners.txt"), encoding="utf8"
-            ) as fdesc:
-                ranges.extend(
-                    (addr, addr, "UK-NCSC") for addr in (line.strip() for line in fdesc)
-                )
-        except FileNotFoundError:
-            LOGGER.warning(
-                "Cannot find file [ukncsc_scanners.txt]. Try running `ivre getwebdata`"
-            )
-        try:
-            with open(
-                os.path.join(DATA_PATH, "censys_scanners.txt"), encoding="utf8"
-            ) as fdesc:
-                ranges.extend(net2range(line.strip()) + ("Censys",) for line in fdesc)
-        except FileNotFoundError:
-            LOGGER.warning(
-                "Cannot find file [censys_scanners.txt]. Try running `ivre getwebdata`"
-            )
-        parsed_ranges = make_range_tables(ranges)
-        _SCANNERS_TABLE = (
-            [elt[0] for elt in parsed_ranges],
-            [elt[1] for elt in parsed_ranges],
-        )
-    if _CDN_TABLE is None:
-        try:
-            with open(
-                os.path.join(DATA_PATH, "cdn_nuclei.py"), encoding="utf8"
-            ) as fdesc:
-                # pylint: disable=eval-used
-                _CDN_TABLE = eval(compile(fdesc.read(), "cdn_nuclei", "eval"))
-        except FileNotFoundError:
-            LOGGER.warning(
-                "Cannot find file [cdn_nuclei.py]. Try running `ivre getwebdata`"
-            )
-            _CDN_TABLE = ([], [])
-
-
-def _get_name(table: Tuple[List[int], List[Optional[str]]], addr: str) -> Optional[str]:
-    """Devs: please make sure _get_data() has been called before calling me!"""
-    addr_i = ip2int(addr) if ":" in addr else ip2int(f"::ffff:{addr}")
-    try:
-        return table[1][bisect_left(table[0], addr_i)]
-    except IndexError:
-        return None
-
-
-def gen_auto_tags(
-    host: NmapHost, update_openports: bool = True
-) -> Generator[Tag, None, None]:
-    """This function generates the automatically-generated tags ("TOR",
-    "Scanner", "Honeypot" and "Vulnerable" / "Likely vulnerable" /
-    "Cannot test vuln", for now).
-
-    If the host has too many (at least `VIEW_SYNACK_HONEYPOT_COUNT`)
-    open ports that may be "syn-ack" honeypots (which means, ports for
-    which is_real_service_port() returns False), this function will
-    generate the "Honeypot" / "SYN+ACK honeypot" tag **and** clean the
-    `host` record from the probable "SYN+ACK honeypot" ports.
-
-    If the "ports" field of the host document has changed, the
-    "openports" field is updated unless `update_openports` is False.
-
-    """
-    _get_data()
-    assert _TOR_NODES is not None
-    assert _SCANNERS_TABLE is not None
-    assert _CDN_TABLE is not None
-    addr = host.get("addr")
-    if isinstance(addr, str):
-        if addr in _TOR_NODES:
-            yield cast(
-                Tag,
-                dict(
-                    TAG_TOR,
-                    info=[
-                        "Exit node listed at <https://check.torproject.org/torbulkexitlist>"
-                    ],
-                ),
-            )
-        cdn_name = _get_name(_CDN_TABLE, addr)
-        if cdn_name is not None:
-            yield cast(
-                Tag,
-                dict(
-                    TAG_CDN,
-                    info=[f"{cdn_name} as listed at <https://cdn.nuclei.sh/>"],
-                ),
-            )
-        scanner_name = _get_name(_SCANNERS_TABLE, addr)
-        if scanner_name is not None:
-            yield cast(
-                Tag,
-                dict(
-                    TAG_SCANNER,
-                    info=[f"Listed as a {scanner_name} scanner"],
-                ),
-            )
-        addr_type = get_addr_type(addr)
-        if addr_type is not None:
-            yield cast(
-                Tag,
-                dict(
-                    TAG_RESERVED,
-                    info=[f"Address type {addr_type}"],
-                ),
-            )
-    for hname in host.get("hostnames", []):
-        name = hname["name"]
-        if name.endswith(".shodan.io") and "census" in name:
-            yield cast(
-                Tag,
-                dict(
-                    TAG_SCANNER,
-                    info=[f"Hostname {name} suggests a Shodan scanner"],
-                ),
-            )
-    for port in host.get("ports", []):
-        if any("honeypot" in port.get(field, "").lower() for field in _SERVICE_FIELDS):
-            cur_info = []
-            for fld in _SERVICE_FIELDS:
-                if fld in port:
-                    cur_info.append(port[fld])
-            yield cast(
-                Tag,
-                dict(
-                    TAG_HONEYPOT,
-                    info=[
-                        f"{' / '.join(cur_info)} on port {port['protocol']}/{port['port']}"
-                    ],
-                ),
-            )
-        if port.get("service_name") in _TOR_SERVICES:
-            yield cast(
-                Tag,
-                dict(
-                    TAG_TOR,
-                    info=[
-                        f"Service {port['service_name']} found on port {port['protocol']}/{port['port']}"
-                    ],
-                ),
-            )
-        elif port.get("service_name") == "ssl":
-            if port.get("service_product") == "Tor over SSL":
-                yield cast(
-                    Tag,
-                    dict(
-                        TAG_TOR,
-                        info=[
-                            f"ssl / {port['service_product']} found on port {port['protocol']}/{port['port']}"
-                        ],
-                    ),
-                )
-        elif port.get("service_name") == "http":
-            if port.get("service_product") in _TOR_HTTP_PRODUCTS:
-                yield cast(
-                    Tag,
-                    dict(
-                        TAG_TOR,
-                        info=[
-                            f"http / {port['service_product']} found on port {port['protocol']}/{port['port']}"
-                        ],
-                    ),
-                )
-        for script in port.get("scripts", []):
-            if script["id"] == "ssl-cert":
-                for cert in script.get("ssl-cert", []):
-                    if cert.get("sha1") in SSLBL_CERTIFICATES:
-                        yield cast(
-                            Tag,
-                            dict(
-                                TAG_MALWARE,
-                                info=[
-                                    f"{SSLBL_CERTIFICATES[cert['sha1']]} certificate on port {port['protocol']}/{port['port']} (SSL Blacklist by abuse.ch)"
-                                ],
-                            ),
-                        )
-                    elif (
-                        TORCERT_SUBJECT.search(cert.get("subject_text", ""))
-                        and TORCERT_SUBJECT.search(cert.get("issuer_text", ""))
-                        and cert.get("subject_text") != cert.get("issuer_text")
-                    ):
-                        yield cast(
-                            Tag,
-                            dict(
-                                TAG_TOR,
-                                info=[
-                                    f"TOR certificate on port {port['protocol']}/{port['port']}"
-                                ],
-                            ),
-                        )
-            elif script["id"] == "ssl-ja3-client":
-                for ja3fp in script.get("ssl-ja3-client", []):
-                    if ja3fp.get("md5") in SSLBL_JA3:
-                        yield cast(
-                            Tag,
-                            dict(
-                                TAG_MALWARE,
-                                info=[
-                                    f"{SSLBL_JA3[ja3fp['md5']]} JA3 client fingerprint (SSL Blacklist by abuse.ch)"
-                                ],
-                            ),
-                        )
-            elif script["id"] == "http-title":
-                tag = {
-                    "This is a Tor Exit Router": dict(
-                        TAG_TOR,
-                        info=[
-                            f"TOR exit node notice on port {port['protocol']}/{port['port']}"
-                        ],
-                    ),
-                    "This is a SOCKS Proxy, Not An HTTP Proxy": dict(
-                        TAG_TOR,
-                        info=[
-                            f"TOR SOCKS Proxy notice on port {port['protocol']}/{port['port']}"
-                        ],
-                    ),
-                    "This is an HTTP CONNECT tunnel, not a full HTTP Proxy": dict(
-                        TAG_TOR,
-                        info=[
-                            f"TOR HTTP CONNECT tunnel notice on port {port['protocol']}/{port['port']}"
-                        ],
-                    ),
-                }.get(script["output"])
-                if tag is not None:
-                    yield cast(Tag, tag)
-            elif script["id"] == "scanner":
-                if port["port"] != -1:
-                    continue
-                scanners = sorted(
-                    set(
-                        scanner["name"]
-                        for scanner in script.get("scanner", {}).get("scanners", [])
-                    )
-                )
-                if scanners:
-                    yield cast(Tag, dict(TAG_SCANNER, info=scanners))
-                else:
-                    yield TAG_SCANNER
-            elif script["id"] == "http-default-accounts":
-                for app in script.get("http-default-accounts", []):
-                    if not app.get("credentials"):
-                        continue
-                    creds = [
-                        f"{cred['username']} / {cred['password']}"
-                        for cred in app["credentials"]
-                    ]
-                    yield cast(
-                        Tag,
-                        dict(
-                            TAG_DEFAULT_PASSWORD,
-                            info=[f"{app['name']}: {', '.join(creds)}"],
-                        ),
-                    )
-            elif "vulns" in script:
-                for vuln in script["vulns"]:
-                    state = vuln.get("state", "")
-                    if state.startswith("VULNERABLE"):
-                        if "id" in vuln:
-                            yield cast(Tag, dict(TAG_VULN, info=[vuln["id"]]))
-                        else:
-                            yield TAG_VULN
-                    elif state.startswith("LIKELY VULNERABLE"):
-                        if "id" in vuln:
-                            yield cast(Tag, dict(TAG_VULN_LIKELY, info=[vuln["id"]]))
-                        else:
-                            yield TAG_VULN_LIKELY
-                    elif state.startswith("UNKNOWN"):
-                        if "id" in vuln:
-                            yield cast(
-                                Tag, dict(TAG_VULN_CANNOT_TEST, info=[vuln["id"]])
-                            )
-                        else:
-                            yield TAG_VULN_CANNOT_TEST
-            elif "nuclei" in script:
-                for template in script["nuclei"]:
-                    template_id = template.get("template", "")
-                    template_name = template.get("name", "")
-                    info = (
-                        template_id
-                        if template_id == template_name
-                        else f"{template_id} / {template_name}"
-                    )
-                    if template_id.startswith("CVE-"):
-                        yield cast(Tag, dict(TAG_VULN, info=[info]))
-                    elif template_name.startswith("CVE-"):
-                        yield cast(Tag, dict(TAG_VULN, info=[info]))
-                    elif any(
-                        template_id.endswith(f"-{suffix}")
-                        for suffix in [
-                            "default-login",
-                            "weak-login",
-                            "weak-password",
-                            "default-admin",
-                        ]
-                    ) or template_id in {
-                        "google-earth-dlogin",
-                        "oracle-business-intelligence-login",
-                        "trilithic-viewpoint-default",
-                    }:
-                        yield cast(Tag, dict(TAG_DEFAULT_PASSWORD, info=[info]))
-                    elif template_name.endswith("Default Password"):
-                        yield cast(Tag, dict(TAG_DEFAULT_PASSWORD, info=[info]))
-    # Now the "Honeypot" / "SYN+ACK honeypot" tag:
-    n_ports = sum(
-        1 for port in host.get("ports", []) if port.get("state_state") == "open"
-    )
-    if n_ports and is_synack_honeypot(host):
-        # 1. If the host is already considered a SYN+ACK honeypot,
-        # let's just clean the ports
-        newports = [
-            port
-            for port in host["ports"]
-            if port.get("state_state") != "open" or is_real_service_port(port)
-        ]
-        if n_ports != sum(1 for port in newports if port.get("state_state") == "open"):
-            host["ports"] = newports
-            if update_openports:
-                set_openports_attribute(host)
-    # 2. ... Else, let's see if we should add the tag
-    elif (
-        VIEW_SYNACK_HONEYPOT_COUNT is not None and n_ports >= VIEW_SYNACK_HONEYPOT_COUNT
-    ):
-        # check if we have too many open ports that could be "syn-ack
-        # honeypots"...
-        newports = [
-            port
-            for port in host["ports"]
-            if port.get("state_state") != "open" or is_real_service_port(port)
-        ]
-        if (
-            n_ports - sum(1 for port in newports if port.get("state_state") == "open")
-            > VIEW_SYNACK_HONEYPOT_COUNT
-        ):
-            # ... if so, keep only the ports that cannot be "syn-ack
-            # honeypots"
-            host["ports"] = newports
-            if update_openports:
-                set_openports_attribute(host)
-        yield cast(Tag, dict(TAG_HONEYPOT, info=["SYN+ACK honeypot"]))
-    # Now the "closed" / "filtered" ports. Note: this won't create any
-    # tag but it is probably the best place to have this code!
-    for status in ["closed", "filtered"]:
-        n_ports = sum(
-            1 for port in host.get("ports", []) if port.get("state_state") == status
-        )
-        if status in host.get("extraports", {}):
-            # 1. If the host already has too many ports in `status`,
-            # let's just update and clean the ports
-            host["extraports"][status]["total"] += n_ports
-            reasons = Counter(
-                port.get("state_reason")
-                for port in host.get("ports", [])
-                if port.get("state_state") == status and port.get("state_reason")
-            )
-            for reason, count in reasons.items():
-                host["extraports"][status].setdefault("reasons", {})[reason] = (
-                    host["extraports"][status].get("reasons", {}).get(reason, 0) + count
-                )
-            if host.get("ports"):
-                host["ports"] = [
-                    port for port in host["ports"] if port.get("state_state") != status
-                ]
-        # 2. ... Else, let's see if we should remove some ports in
-        # `status`.
-        elif (
-            VIEW_SYNACK_HONEYPOT_COUNT is not None
-            and n_ports >= VIEW_SYNACK_HONEYPOT_COUNT
-        ):
-            host.setdefault("extraports", {})[status] = {"total": n_ports}
-            reasons = Counter(
-                port.get("state_reason")
-                for port in host.get("ports", [])
-                if port.get("state_state") == status and port.get("state_reason")
-            )
-            for reason, count in reasons.items():
-                host["extraports"][status].setdefault("reasons", {})[reason] = count
-            if host.get("ports"):
-                host["ports"] = [
-                    port for port in host["ports"] if port.get("state_state") != status
-                ]
-
-
-def set_auto_tags(host: NmapHost, update_openports: bool = True) -> None:
-    """This function sets the automatically-generated tags ("TOR",
-    "Scanner", "Honeypot" and "Vulnerable" / "Likely vulnerable" /
-    "Cannot test vuln", for now).
-
-    """
-    add_tags(host, gen_auto_tags(host, update_openports=update_openports))
-
-
 def merge_ja3_scripts(
     curscript: NmapScript, script: NmapScript, script_id: str
 ) -> NmapScript:
@@ -851,6 +175,18 @@ def merge_ja3_scripts(
     return _merge_scripts(curscript, script, script_id, ja3_equals, ja3_output)
 
 
+def merge_ja4_scripts(
+    curscript: NmapScript, script: NmapScript, script_id: str
+) -> NmapScript:
+    def ja4_equals(a: Dict[str, Any], b: Dict[str, Any], script_id: str) -> bool:
+        return cast(bool, a["ja4"] == b["ja4"])
+
+    def ja4_output(ja4: Dict[str, Any], script_id: str) -> str:
+        return cast(str, ja4["ja4"])
+
+    return _merge_scripts(curscript, script, script_id, ja4_equals, ja4_output)
+
+
 def merge_http_app_scripts(
     curscript: NmapScript, script: NmapScript, script_id: str
 ) -> NmapScript:
@@ -1151,6 +487,38 @@ def merge_dns_domains_scripts(
     return curscript
 
 
+def merge_dns_domains_mx_scripts(
+    curscript: NmapScript, script: NmapScript, script_id: str
+) -> NmapScript:
+    domains_prios = {
+        (res["domain"], res["priority"]): (
+            res["parents"] if "parents" in res else list(get_domains(res["domain"]))
+        )
+        for scr in [script, curscript]
+        for res in scr.get(script_id, [])
+        if "domain" in res and "priority" in res
+    }
+    domains_order = sorted(
+        domains_prios, key=lambda dom_prio: (key_sort_dom(dom_prio[0]), dom_prio[1])
+    )
+    if len(domains_order) == 1:
+        output = f"Server is Mail eXchanger for {domains_order[0][0]} (priority {domains_order[0][1]})"
+    else:
+        output = "Server is Mail eXchanger for:\n%s" % "\n".join(
+            f"  {dom} (priority {prio})" for dom, prio in domains_order
+        )
+    curscript["output"] = output
+    curscript[script_id] = [
+        {
+            "domain": dom_prio[0],
+            "parents": domains_prios[dom_prio],
+            "priority": dom_prio[1],
+        }
+        for dom_prio in domains_order
+    ]
+    return curscript
+
+
 def merge_dns_tls_rpt_scripts(
     curscript: NmapScript, script: NmapScript, script_id: str
 ) -> NmapScript:
@@ -1264,6 +632,7 @@ def _merge_scripts(
 _SCRIPT_MERGE = {
     "dns-check-consistency": merge_dns_check_consistency_scripts,
     "dns-domains": merge_dns_domains_scripts,
+    "dns-domains-mx": merge_dns_domains_mx_scripts,
     "dns-tls-rpt": merge_dns_tls_rpt_scripts,
     "dns-zone-transfer": merge_axfr_scripts,
     "http-app": merge_http_app_scripts,
@@ -1276,6 +645,7 @@ _SCRIPT_MERGE = {
     "ssl-cert": merge_ssl_cert_scripts,
     "ssl-ja3-client": merge_ja3_scripts,
     "ssl-ja3-server": merge_ja3_scripts,
+    "ssl-ja4-client": merge_ja4_scripts,
 }
 
 
@@ -1290,11 +660,20 @@ def merge_scripts(
     return func(curscript, script, script_id)
 
 
-def merge_host_docs(rec1: NmapHost, rec2: NmapHost) -> NmapHost:
+def merge_host_docs(
+    rec1: NmapHost,
+    rec2: NmapHost,
+    auto_tags: bool = True,
+    openports_attribute: bool = True,
+) -> NmapHost:
     """Merge two host records and return the result. Unmergeable /
     hard-to-merge fields are lost (e.g., extraports).
 
     """
+    if not rec2:
+        return rec1
+    if not rec1:
+        return rec2
     if rec1.get("schema_version") != rec2.get("schema_version"):
         raise ValueError(
             "Cannot merge host documents. "
@@ -1325,14 +704,16 @@ def merge_host_docs(rec1: NmapHost, rec2: NmapHost) -> NmapHost:
     rec["state_reason"] = rec2.get("state_reason", rec1.get("state_reason"))
     if rec["state_reason"] is None:
         del rec["state_reason"]
-    rec["categories"] = list(
+    rec["categories"] = sorted(
         set(rec1.get("categories", [])).union(rec2.get("categories", []))
     )
     for field in ["addr", "os"]:
         rec[field] = rec2[field] if rec2.get(field) else rec1.get(field)
         if not rec[field]:
             del rec[field]
-    rec["source"] = list(set(rec1.get("source", [])).union(set(rec2.get("source", []))))
+    rec["source"] = sorted(
+        set(rec1.get("source", [])).union(set(rec2.get("source", [])))
+    )
     rec["traces"] = rec2.get("traces", [])
     for trace in rec1.get("traces", []):
         # Skip this result (from rec1) if a more recent traceroute
@@ -1362,14 +743,18 @@ def merge_host_docs(rec1: NmapHost, rec2: NmapHost) -> NmapHost:
     rec["infos"] = {}
     for record in [rec1, rec2]:
         rec["infos"].update(record.get("infos", {}))
-    # We want to make sure of (type, name) unicity
-    hostnames = dict(
-        ((h["type"], h["name"]), h.get("domains"))
-        for h in (rec1.get("hostnames", []) + rec2.get("hostnames", []))
-    )
-    rec["hostnames"] = [
-        {"type": h[0], "name": h[1], "domains": d} for h, d in hostnames.items()
-    ]
+    if not (has_toomany_hostnames(rec1) or has_toomany_hostnames(rec2)):
+        # We want to make sure of (type, name) unicity
+        hostnames = {
+            (h["type"], h["name"]): h.get("domains")
+            for h in chain(rec1.get("hostnames", []), rec2.get("hostnames", []))
+        }
+        if VIEW_MAX_HOSTNAMES_COUNT and len(hostnames) > VIEW_MAX_HOSTNAMES_COUNT:
+            add_tags(rec, [cast(Tag, dict(TAG_CDN, info=["Too many hostnames"]))])
+        elif hostnames:
+            rec["hostnames"] = [
+                {"type": h[0], "name": h[1], "domains": d} for h, d in hostnames.items()
+            ]
     addresses: NmapAddress = {}
     for record in [rec1, rec2]:
         for atype, addrs in record.get("addresses", {}).items():
@@ -1383,10 +768,6 @@ def merge_host_docs(rec1: NmapHost, rec2: NmapHost) -> NmapHost:
     if addresses:
         rec["addresses"] = addresses
     if sa_honeypot:
-        add_tags(
-            rec,
-            [{"value": "Honeypot", "type": "warning", "info": ["SYN+ACK honeypot"]}],
-        )
         for record in [rec1, rec2]:
             if not is_synack_honeypot(record):
                 record["ports"] = [
@@ -1398,10 +779,10 @@ def merge_host_docs(rec1: NmapHost, rec2: NmapHost) -> NmapHost:
         rec["extraports"] = rec1["extraports"]
     elif "extraports" in rec2 and "extraports" not in rec1:
         rec["extraports"] = rec2["extraports"]
-    ports = dict(
-        ((port.get("protocol"), port["port"]), port.copy())
+    ports = {
+        (port.get("protocol"), port["port"]): port.copy()
         for port in rec2.get("ports", [])
-    )
+    }
     for port in rec1.get("ports", []):
         if (port.get("protocol"), port["port"]) in ports:
             curport = ports[(port.get("protocol"), port["port"])]
@@ -1447,8 +828,38 @@ def merge_host_docs(rec1: NmapHost, rec2: NmapHost) -> NmapHost:
             port.get("port"),
         ),
     )
-    set_auto_tags(rec, update_openports=False)
-    set_openports_attribute(rec)
+    if auto_tags:
+        set_auto_tags(rec, update_openports=False)
+    else:
+        # we at least need to clean-up the ports
+        # first: syn-ack honeypot
+        n_ports = sum(
+            1 for port in rec.get("ports", []) if port.get("state_state") == "open"
+        )
+        if (
+            VIEW_SYNACK_HONEYPOT_COUNT is not None
+            and n_ports >= VIEW_SYNACK_HONEYPOT_COUNT
+        ):
+            # check if we have too many open ports that could be
+            # "syn-ack honeypots"...
+            newports = [
+                port
+                for port in rec["ports"]
+                if port.get("state_state") != "open" or is_real_service_port(port)
+            ]
+            if (
+                n_ports
+                - sum(1 for port in newports if port.get("state_state") == "open")
+                > VIEW_SYNACK_HONEYPOT_COUNT
+            ):
+                # ... if so, keep only the ports that cannot be "syn-ack
+                # honeypots"
+                rec["ports"] = newports
+                add_tags(
+                    rec, [cast(Tag, dict(TAG_HONEYPOT, info=["SYN+ACK honeypot"]))]
+                )
+    if openports_attribute:
+        set_openports_attribute(rec)
     for field in ["traces", "infos", "ports", "cpes"]:
         if not rec[field]:
             del rec[field]
@@ -1504,9 +915,9 @@ def create_http_ls(data: bytes, volname: str = "???") -> Optional[NmapScript]:
         for i, t in enumerate(title[:-1]):
             column_width[i] = max(column_width[i], len(fobj.get(t, "-")))
     line_fmt = "%%(size)-%ds  %%(time)-%ds  %%(filename)s" % tuple(column_width)
-    output.append(line_fmt % dict((t, t.upper()) for t in title))
+    output.append(line_fmt % {t: t.upper() for t in title})
     for fobj in files:
-        output.append(line_fmt % dict({"size": "-", "time": "-"}, **fobj))
+        output.append(line_fmt % {"size": "-", "time": "-", **fobj})
     output.append("")
     return {
         "id": "http-ls",
@@ -1616,7 +1027,9 @@ def handle_http_content(
     path: str = "/",
 ) -> None:
     title_m = _EXPR_TITLE.search(data)
-    if title_m is not None:
+    if title_m is not None and not any(
+        s["id"] == "http-title" for s in port.get("scripts", [])
+    ):
         title = nmap_encode_data(title_m.groups()[0])
         port.setdefault("scripts", []).append(
             {
@@ -1746,3 +1159,6 @@ def handle_http_headers(
             port.setdefault("scripts", []).append(script)
         else:
             merge_http_app_scripts(cur_script, script, "http-app")
+
+
+load_plugins("ivre.plugins.active.data", globals())
