@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of IVRE.
-# Copyright 2011 - 2021 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2023 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -56,9 +56,8 @@ WEB_STATIC_PATH = None
 WEB_DOKU_PATH = None
 AGENT_MASTER_PATH = "/var/lib/ivre/master"
 # specific: if no value is specified, tries /usr/local/share/nmap,
-# /opt/nmap/share/nmap, then /usr/share/nmap; same for wireshark.
+# /opt/nmap/share/nmap, then /usr/share/nmap.
 NMAP_SHARE_PATH = None
-WIRESHARK_SHARE_PATH = None
 # Begin commands
 TESSERACT_CMD = "tesseract"
 OPENSSL_CMD = "openssl"
@@ -255,6 +254,9 @@ GEOIP_LANG = "en"
 # situations. None means that no host should be treated as a "syn-ack
 # honeypot".
 VIEW_SYNACK_HONEYPOT_COUNT = 200
+# Some hostnames (often at CDNs) have too many hostnames and it might
+# not be very useful to count them all
+VIEW_MAX_HOSTNAMES_COUNT = 200
 
 WEB_ALLOWED_REFERERS = None
 WEB_NOTES_BASE = "/dokuwiki/#IP#"
@@ -376,7 +378,3 @@ if HONEYD_IVRE_SCRIPTS_PATH is None and DATA_PATH is not None:
 
 if NMAP_SHARE_PATH is None:
     NMAP_SHARE_PATH = guess_share("nmap")
-
-
-if WIRESHARK_SHARE_PATH is None:
-    WIRESHARK_SHARE_PATH = guess_share("wireshark")
