@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of IVRE.
-# Copyright 2011 - 2022 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2024 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -31,14 +31,14 @@ from elasticsearch_dsl import Q
 from elasticsearch_dsl.query import Query
 
 from ivre import utils
-from ivre.active.data import ALIASES_TABLE_ELEMS
+from ivre.active.nmap import ALIASES_TABLE_ELEMS
 from ivre.db import DB, DBActive, DBView
+from ivre.plugins import load_plugins
 
 PAGESIZE = 250
 
 
 class ElasticDB(DB):
-
     nested_fields = []
 
     # filters
@@ -49,6 +49,7 @@ class ElasticDB(DB):
         self.username = ""
         self.password = ""
         self.hosts = None
+        self.tls = url.scheme == "elastics"
         if "@" in url.netloc:
             username, hostname = url.netloc.split("@", 1)
             if ":" in username:
@@ -58,9 +59,9 @@ class ElasticDB(DB):
             else:
                 self.username = unquote(username)
             if hostname:
-                self.hosts = [hostname]
+                self.hosts = [f"http{'s' if self.tls else ''}://{hostname}"]
         elif url.netloc:
-            self.hosts = [url.netloc]
+            self.hosts = [f"http{'s' if self.tls else ''}://{url.netloc}"]
         index_prefix = url.path.lstrip("/")
         if index_prefix:
             self.index_prefix = index_prefix + "-"
@@ -231,7 +232,6 @@ def _create_mappings(nested, all_mappings):
 
 
 class ElasticDBActive(ElasticDB, DBActive):
-
     nested_fields = [
         "ports",
         "ports.scripts",
@@ -240,6 +240,7 @@ class ElasticDBActive(ElasticDB, DBActive):
         "ports.scripts.ssl-cert",
         "ports.scripts.ssl-ja3-client",
         "ports.scripts.ssl-ja3-server",
+        "ports.scripts.ssl-ja4-client",
         "tags",
     ]
     mappings = [
@@ -308,7 +309,7 @@ class ElasticDBActive(ElasticDB, DBActive):
     def distinct(self, field, flt=None, sort=None, limit=None, skip=None):
         if flt is None:
             flt = self.flt_empty
-        if field == "infos.coordinates":
+        if field == "infos.coordinates" and hasattr(self, "searchhaslocation"):
 
             def fix_result(value):
                 return tuple(float(v) for v in value.split(", "))
@@ -347,42 +348,8 @@ class ElasticDBActive(ElasticDB, DBActive):
                 break
             query["after"] = result["aggregations"]["values"]["after_key"]
 
-    def getlocations(self, flt):
-        query = {
-            "size": PAGESIZE,
-            "sources": [
-                {
-                    "coords": {
-                        "terms": {
-                            "script": {
-                                "lang": "painless",
-                                "source": "doc['infos.coordinates'].value",
-                            }
-                        }
-                    }
-                }
-            ],
-        }
-        flt = self.flt_and(flt & self.searchhaslocation())
-        while True:
-            result = self.db_client.search(
-                body={"query": flt.to_dict(), "aggs": {"values": {"composite": query}}},
-                index=self.indexes[0],
-                ignore_unavailable=True,
-                size=0,
-            )
-            for value in result["aggregations"]["values"]["buckets"]:
-                yield {
-                    "_id": tuple(float(v) for v in value["key"]["coords"].split(", ")),
-                    "count": value["doc_count"],
-                }
-            if "after_key" not in result["aggregations"]["values"]:
-                break
-            query["after"] = result["aggregations"]["values"]["after_key"]
-
     def topvalues(self, field, flt=None, topnbr=10, sort=None, least=False):
-        """
-        This method uses an aggregation to produce top values for a given
+        """This method uses an aggregation to produce top values for a given
         field or pseudo-field. Pseudo-fields are:
           - category[:regexp] / asnum / country / net[:mask]
           - port
@@ -406,9 +373,11 @@ class ElasticDBActive(ElasticDB, DBActive):
           - hop
           - scanner.name / scanner.port:tcp / scanner.port:udp
           - domains / domains[:level] / domains[:domain] / domains[:domain[:level]]
-          - ja3-client[:filter][.type], ja3-server[:filter][:client][.type], jarm
+          - ja3-client[:filter][.type], ja3-server[:filter][:client][.type]
+          - ja4-client[:filter][.type], jarm
           - hassh.type, hassh-client.type, hassh-server.type
           - tag.{value,type,info} / tag[:value]
+
         """
         baseterms = {"size": topnbr}
         if least:
@@ -432,7 +401,7 @@ class ElasticDBActive(ElasticDB, DBActive):
             field = {"field": "infos.as_num"}
         elif field == "as":
 
-            def outputproc(value):
+            def outputproc(value):  # noqa: F811
                 return tuple(
                     val if i else int(val) for i, val in enumerate(value.split(",", 1))
                 )
@@ -984,7 +953,8 @@ return result;
             if subkey is not None:
                 if subkey != subfield:
                     base = {
-                        "filter": filter_value,
+                        # filter_value exists when subkey is not None
+                        "filter": filter_value,  # pylint: disable=possibly-used-before-assignment
                         "aggs": {"patterns": base},
                     }
                 else:
@@ -1093,12 +1063,14 @@ return result;
             }
             if value1 is not None:
                 base = {
-                    "filter": filter_value1,
+                    # filter_value1 exists when value1 is not None
+                    "filter": filter_value1,  # pylint: disable=used-before-assignment
                     "aggs": {"patterns": base},
                 }
             if value2 is not None:
                 base = {
-                    "filter": filter_value2,
+                    # filter_value2 exists when value2 is not None
+                    "filter": filter_value2,  # pylint: disable=used-before-assignment
                     "aggs": {"patterns": base},
                 }
             flt = self.flt_and(
@@ -1122,6 +1094,45 @@ return result;
                     }
                 },
             }
+        elif field == "ja4-client" or (
+            field.startswith("ja4-client") and field[10] in ":."
+        ):
+            if ":" in field:
+                field, value = field.split(":", 1)
+                if isinstance(value, utils.REGEXP_T):
+                    include_value = self._get_pattern(value)
+                else:
+                    include_value = re.escape(value)
+            else:
+                value = None
+                include_value = None
+            if "." in field:
+                field, subfield = field.split(".", 1)
+            else:
+                subfield = "ja4"
+            base = {
+                "terms": dict(
+                    baseterms,
+                    field="ports.scripts.ssl-ja4-client.%s" % subfield,
+                ),
+            }
+            if include_value is not None:
+                base["terms"]["include"] = include_value
+            flt = self.flt_and(flt, self.searchja4client(value=value))
+            nested = {
+                "nested": {"path": "ports"},
+                "aggs": {
+                    "patterns": {
+                        "nested": {"path": "ports.scripts"},
+                        "aggs": {
+                            "patterns": {
+                                "nested": {"path": "ports.scripts.ssl-ja4-client"},
+                                "aggs": {"patterns": base},
+                            }
+                        },
+                    }
+                },
+            }
         elif field == "hassh" or (field.startswith("hassh") and field[5] in "-."):
             if "." in field:
                 field, subfield = field.split(".", 1)
@@ -1214,7 +1225,7 @@ return result;
                     }
                 },
             }
-        elif field == "tag":
+        elif field == "tag" and hasattr(self, "searchtag"):
             flt = self.flt_and(flt, self.searchtag())
 
             def outputproc(value):
@@ -1234,10 +1245,10 @@ return result;
                     }
                 },
             }
-        elif field.startswith("tag."):
+        elif field.startswith("tag.") and hasattr(self, "searchtag"):
             flt = self.flt_and(flt, self.searchtag())
             field = {"field": f"tags.{field[4:]}"}
-        elif field.startswith("tag:"):
+        elif field.startswith("tag:") and hasattr(self, "searchtag"):
             subfield = field[4:]
             flt = self.flt_and(flt, self.searchtag(tag={"value": subfield}))
             nested = {
@@ -1298,86 +1309,6 @@ return result;
             return ~res
         return res
 
-    @classmethod
-    def searchtag(cls, tag=None, neg=False):
-        """Filters (if `neg` == True, filters out) one particular tag (records
-        may have zero, one or more tags).
-
-        `tag` may be the value (as a str) or the tag (as a Tag, e.g.:
-        `{"value": value, "info": info}`).
-
-        """
-        if not tag:
-            res = Q("exists", field="tags.value")
-            if neg:
-                return ~res
-            return res
-        if not isinstance(tag, dict):
-            tag = {"value": tag}
-        all_res = []
-        for key, value in tag.items():
-            if isinstance(value, list) and len(value) == 1:
-                value = value[0]
-            if isinstance(value, list):
-                res = Q("terms", **{f"tags.{key}": value})
-            elif isinstance(value, utils.REGEXP_T):
-                res = Q("regexp", **{f"tags.{key}": cls._get_pattern(value)})
-            else:
-                res = Q("match", **{f"tags.{key}": value})
-            if neg:
-                all_res.append(~res)
-            else:
-                all_res.append(res)
-        if neg:
-            return cls.flt_or(
-                ~Q("exists", field="tags.value"),
-                Q("nested", path="tags", query=cls.flt_or(*all_res)),
-            )
-        return Q("nested", path="tags", query=cls.flt_and(*all_res))
-
-    @staticmethod
-    def searchcountry(country, neg=False):
-        """Filters (if `neg` == True, filters out) one particular
-        country, or a list of countries.
-
-        """
-        country = utils.country_unalias(country)
-        if isinstance(country, list):
-            res = Q("terms", infos__country_code=country)
-        else:
-            res = Q("match", infos__country_code=country)
-        if neg:
-            return ~res
-        return res
-
-    @staticmethod
-    def searchasnum(asnum, neg=False):
-        """Filters (if `neg` == True, filters out) one or more
-        particular AS number(s).
-
-        """
-        if not isinstance(asnum, str) and hasattr(asnum, "__iter__"):
-            res = Q("terms", infos__as_num=[int(val) for val in asnum])
-        else:
-            res = Q("match", infos__as_num=int(asnum))
-        if neg:
-            return ~res
-        return res
-
-    @classmethod
-    def searchasname(cls, asname, neg=False):
-        """Filters (if `neg` == True, filters out) one or more
-        particular AS.
-
-        """
-        if isinstance(asname, utils.REGEXP_T):
-            res = Q("regexp", infos__as_name=cls._get_pattern(asname))
-        else:
-            res = Q("match", infos__as_name=asname)
-        if neg:
-            return ~res
-        return res
-
     @staticmethod
     def searchopenport(neg=False):
         "Filters records with at least one open port."
@@ -1636,9 +1567,9 @@ return result;
                         Q(
                             "match",
                             **{
-                                "ports.scripts.id": "ssl-cacert"
-                                if cacert
-                                else "ssl-cert"
+                                "ports.scripts.id": (
+                                    "ssl-cacert" if cacert else "ssl-cert"
+                                )
                             },
                         ),
                         Q(
@@ -1714,3 +1645,119 @@ class ElasticDBView(ElasticDBActive, DBView):
     def store_or_merge_host(self, host):
         if not self.merge_host(host):
             self.store_host(host)
+
+    @classmethod
+    def searchtag(cls, tag=None, neg=False):
+        """Filters (if `neg` == True, filters out) one particular tag (records
+        may have zero, one or more tags).
+
+        `tag` may be the value (as a str) or the tag (as a Tag, e.g.:
+        `{"value": value, "info": info}`).
+
+        """
+        if not tag:
+            res = Q("exists", field="tags.value")
+            if neg:
+                return ~res
+            return res
+        if not isinstance(tag, dict):
+            tag = {"value": tag}
+        all_res = []
+        for key, value in tag.items():
+            if isinstance(value, list) and len(value) == 1:
+                value = value[0]
+            if isinstance(value, list):
+                res = Q("terms", **{f"tags.{key}": value})
+            elif isinstance(value, utils.REGEXP_T):
+                res = Q("regexp", **{f"tags.{key}": cls._get_pattern(value)})
+            else:
+                res = Q("match", **{f"tags.{key}": value})
+            if neg:
+                all_res.append(~res)
+            else:
+                all_res.append(res)
+        if neg:
+            return cls.flt_or(
+                ~Q("exists", field="tags.value"),
+                Q("nested", path="tags", query=cls.flt_or(*all_res)),
+            )
+        return Q("nested", path="tags", query=cls.flt_and(*all_res))
+
+    @staticmethod
+    def searchcountry(country, neg=False):
+        """Filters (if `neg` == True, filters out) one particular
+        country, or a list of countries.
+
+        """
+        country = utils.country_unalias(country)
+        if isinstance(country, list):
+            res = Q("terms", infos__country_code=country)
+        else:
+            res = Q("match", infos__country_code=country)
+        if neg:
+            return ~res
+        return res
+
+    @staticmethod
+    def searchasnum(asnum, neg=False):
+        """Filters (if `neg` == True, filters out) one or more
+        particular AS number(s).
+
+        """
+        if not isinstance(asnum, str) and hasattr(asnum, "__iter__"):
+            res = Q("terms", infos__as_num=[int(val) for val in asnum])
+        else:
+            res = Q("match", infos__as_num=int(asnum))
+        if neg:
+            return ~res
+        return res
+
+    @classmethod
+    def searchasname(cls, asname, neg=False):
+        """Filters (if `neg` == True, filters out) one or more
+        particular AS.
+
+        """
+        if isinstance(asname, utils.REGEXP_T):
+            res = Q("regexp", infos__as_name=cls._get_pattern(asname))
+        else:
+            res = Q("match", infos__as_name=asname)
+        if neg:
+            return ~res
+        return res
+
+    def getlocations(self, flt):
+        query = {
+            "size": PAGESIZE,
+            "sources": [
+                {
+                    "coords": {
+                        "terms": {
+                            "script": {
+                                "lang": "painless",
+                                "source": "doc['infos.coordinates'].value",
+                            }
+                        }
+                    }
+                }
+            ],
+        }
+        flt = self.flt_and(flt & self.searchhaslocation())
+        while True:
+            result = self.db_client.search(
+                body={"query": flt.to_dict(), "aggs": {"values": {"composite": query}}},
+                index=self.indexes[0],
+                ignore_unavailable=True,
+                size=0,
+            )
+            for value in result["aggregations"]["values"]["buckets"]:
+                yield {
+                    "_id": tuple(float(v) for v in value["key"]["coords"].split(", ")),
+                    "count": value["doc_count"],
+                }
+            if "after_key" not in result["aggregations"]["values"]:
+                break
+            query["after"] = result["aggregations"]["values"]["after_key"]
+
+
+load_plugins("ivre.plugins.db.elastic", globals())
