@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of IVRE.
-# Copyright 2011 - 2022 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2024 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -35,7 +35,6 @@ from sqlalchemy import (
     ARRAY,
     Column,
     Index,
-    LargeBinary,
     String,
     Table,
     and_,
@@ -48,7 +47,7 @@ from sqlalchemy import (
     insert,
     join,
     not_,
-    nullsfirst,
+    nulls_first,
     select,
     text,
     tuple_,
@@ -395,9 +394,11 @@ class PostgresDBActive(PostgresDB, SQLDBActive):
             field = self._topstructure(
                 self.tables.port,
                 [self.tables.port.protocol, self.tables.port.port],
-                (self.tables.port.state == info)
-                if info in ["open", "filtered", "closed", "open|filtered"]
-                else (self.tables.port.service_name == info),
+                (
+                    (self.tables.port.state == info)
+                    if info in ["open", "filtered", "closed", "open|filtered"]
+                    else (self.tables.port.service_name == info)
+                ),
             )
         elif field.startswith("countports:"):
             info = field[11:]
@@ -747,7 +748,9 @@ class PostgresDBActive(PostgresDB, SQLDBActive):
             field = self._topstructure(
                 self.tables.script,
                 [
-                    func.jsonb_array_elements(self.tables.script.data["ssl-cert"],).op(
+                    func.jsonb_array_elements(
+                        self.tables.script.data["ssl-cert"],
+                    ).op(
                         "->" if subfield in ["subject", "issuer", "pubkey"] else "->>"
                     )(subfield)
                 ],
@@ -812,6 +815,8 @@ class PostgresDBActive(PostgresDB, SQLDBActive):
                     [column("ssl_ja3_client").op("->>")(subfield)],
                     and_(
                         self.tables.script.name == "ssl-ja3-client",
+                        # when value is not None, subkey is defined
+                        # pylint: disable=possibly-used-before-assignment
                         self._searchstring_re(
                             column("ssl_ja3_client").op("->>")(subkey),
                             value,
@@ -987,9 +992,11 @@ class PostgresDBActive(PostgresDB, SQLDBActive):
             field = self._topstructure(
                 self.tables.hop,
                 [self.tables.hop.ipaddr],
-                (self.tables.hop.ttl > ttl)
-                if field[3] == ">"
-                else (self.tables.hop.ttl == ttl),
+                (
+                    (self.tables.hop.ttl > ttl)
+                    if field[3] == ">"
+                    else (self.tables.hop.ttl == ttl)
+                ),
             )
         elif field == "file" or (field.startswith("file") and field[4] in ".:"):
             if field.startswith("file:"):
@@ -1145,21 +1152,21 @@ class PostgresDBActive(PostgresDB, SQLDBActive):
                     ),
                 ),
             )
-        elif field == "tag":
+        elif field == "tag" and hasattr(self, "searchtag"):
             flt = self.flt_and(flt, self.searchtag())
             field = self._topstructure(
                 self.tables.tag, [self.tables.tag.value, self.tables.tag.info]
             )
-        elif field == "tag.value":
+        elif field == "tag.value" and hasattr(self, "searchtag"):
             flt = self.flt_and(flt, self.searchtag())
             field = self._topstructure(self.tables.tag, [self.tables.tag.value])
-        elif field == "tag.info":
+        elif field == "tag.info" and hasattr(self, "searchtag"):
             flt = self.flt_and(flt, self.searchtag())
             field = self._topstructure(self.tables.tag, [self.tables.tag.info])
-        elif field == "tag.type":
+        elif field == "tag.type" and hasattr(self, "searchtag"):
             flt = self.flt_and(flt, self.searchtag())
             field = self._topstructure(self.tables.tag, [self.tables.tag.type])
-        elif field.startswith("tag:"):
+        elif field.startswith("tag:") and hasattr(self, "searchtag"):
             subfield = field[4:]
             flt = self.flt_and(flt, self.searchtag(tag={"value": subfield}))
             field = self._topstructure(
@@ -1260,7 +1267,7 @@ class PostgresDBActive(PostgresDB, SQLDBActive):
             )
         )
         if not yieldall:
-            req = req.order_by(*(nullsfirst(fld) for fld in fields))
+            req = req.order_by(*(nulls_first(fld) for fld in fields))
             return self.db.execute(req)
         # results will be modified, we cannot keep a RowProxy
         # instance, so we convert the results to lists
@@ -1360,48 +1367,6 @@ class PostgresDBActive(PostgresDB, SQLDBActive):
 
 
 class PostgresDBNmap(PostgresDBActive, SQLDBNmap):
-    def store_scan_doc(self, scan):
-        scan = scan.copy()
-        if "start" in scan:
-            scan["start"] = datetime.datetime.utcfromtimestamp(int(scan["start"]))
-        if "scaninfos" in scan:
-            scan["scaninfo"] = scan.pop("scaninfos")
-        scan["sha256"] = utils.decode_hex(scan.pop("_id"))
-        insrt = insert(self.tables.scanfile).values(
-            **dict(
-                (key, scan[key])
-                for key in [
-                    "sha256",
-                    "args",
-                    "scaninfo",
-                    "scanner",
-                    "start",
-                    "version",
-                    "xmloutputversion",
-                ]
-                if key in scan
-            )
-        )
-        if config.DEBUG:
-            scanfileid = self.db.execute(
-                insrt.returning(self.tables.scanfile.sha256)
-            ).fetchone()[0]
-            utils.LOGGER.debug("SCAN STORED: %r", utils.encode_hex(scanfileid))
-        else:
-            self.db.execute(insrt)
-
-    def update_scan_doc(self, scan_id, data):
-        data = data.copy()
-        if "end" in data:
-            data["end"] = datetime.datetime.utcfromtimestamp(int(data["end"]))
-        self.db.execute(
-            update(self.tables.scanfile)
-            .where(self.tables.scanfile.sha256 == utils.decode_hex(scan_id))
-            .values(
-                **dict((key, data[key]) for key in ["end", "elapsed"] if key in data)
-            )
-        )
-
     def _store_host(self, host):
         addr = self.ip2internal(host["addr"])
         info = host.get("infos")
@@ -1516,19 +1481,12 @@ class PostgresDBNmap(PostgresDBActive, SQLDBNmap):
         return scanid
 
     def store_host(self, host):
-        scanid = self._store_host(host)
-        insrt = postgresql.insert(self.tables.association_scan_scanfile)
-        self.db.execute(
-            insrt.values(
-                scan=scanid, scan_file=utils.decode_hex(host["scanid"])
-            ).on_conflict_do_nothing()
-        )
+        self._store_host(host)
 
     def store_hosts(self, hosts):
         tmp = self.create_tmp_table(
             self.tables.scan,
             extracols=[
-                Column("scanfileid", ARRAY(LargeBinary(32))),
                 Column("categories", ARRAY(String(32))),
                 Column("source", String(32)),
                 # Column("cpe", postgresql.JSONB),
@@ -1559,11 +1517,11 @@ class PostgresDBView(PostgresDBActive, SQLDBView):
                 info=info,
                 time_start=host_tstart,
                 time_stop=host_tstop,
-                **dict(
-                    (key, host.get(key))
+                **{
+                    key: host.get(key)
                     for key in ["state", "state_reason", "state_reason_ttl"]
                     if key in host
-                ),
+                },
             )
             .on_conflict_do_update(
                 index_elements=["addr"],
@@ -1615,7 +1573,7 @@ class PostgresDBView(PostgresDBActive, SQLDBView):
                 insrt.values(scan=scanid, **port)
                 .on_conflict_do_update(
                     index_elements=["scan", "port", "protocol"],
-                    set_=dict(scan=scanid, **(port if newest else {})),
+                    set_={"scan": scanid, **(port if newest else {})},
                 )
                 .returning(self.tables.port.id)
             ).fetchone()[0]
