@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of IVRE.
-# Copyright 2011 - 2022 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2024 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -70,7 +70,7 @@ class Target:
         if categories is None:
             self.categories = [self.name]
         else:
-            self.categories = categories
+            self.categories = sorted(set(categories))
         self.infos = {"categories": self.categories}
 
     def __len__(self):
@@ -126,7 +126,7 @@ class IterTarget:
             # a - 1 is divisible by all prime factors of m
             mfactors = reduce(mul, set(mathutils.factors(self.lcg_m)))
             # a - 1 is a multiple of 4 if m is a multiple of 4.
-            if self.lcg_m % 4 == 0:
+            if not self.lcg_m % 4:
                 mfactors *= 2
             self.lcg_a = mfactors + 1
         else:
@@ -342,6 +342,8 @@ class TargetFile(Target):
         self.name = "FILE-%s" % filename
         if categories is None:
             categories = [self.name]
+        else:
+            categories = sorted(set(categories))
         self.infos = {"categories": categories}
         with open(filename, encoding="utf8") as fdesc:
             self.targetscount = sum(
@@ -387,7 +389,7 @@ class IterTargetFile:
 
     def __readline__(self):
         line = self.fdesc.readline()
-        if line == "":
+        if not line:
             self.fdesc.close()
             self.target.close()
             raise StopIteration
@@ -443,7 +445,11 @@ class TargetNmapPreScan(TargetZMapPreScan):
 
     @classmethod
     def _getaddr(cls, line):
-        addr = cls.match_addr.match(line)
+        try:
+            line_s = line.decode()
+        except UnicodeDecodeError:
+            return None
+        addr = cls.match_addr.match(line_s)
         if addr is not None:
             try:
                 return utils.ip2int(addr.groups()[0])
