@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of IVRE.
-# Copyright 2011 - 2022 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2024 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@ import shutil
 import socket
 import struct
 import subprocess
+import sys
 import time
 from bisect import bisect_left
 from io import BytesIO
@@ -62,16 +63,17 @@ from typing import (
     Set,
     Tuple,
     Type,
+    TypeVar,
     Union,
     cast,
 )
-from urllib.error import HTTPError
+from urllib.error import HTTPError, URLError
 from urllib.parse import urlparse
 from urllib.request import build_opener
 
 try:
     from cryptography.hazmat.primitives.serialization import Encoding, PublicFormat
-    from OpenSSL import crypto as osslc  # type: ignore
+    from OpenSSL import crypto as osslc
 except ImportError:
     USE_PYOPENSSL = False
 else:
@@ -83,6 +85,12 @@ except ImportError:
     USE_PIL = False
 else:
     USE_PIL = True
+try:
+    from bson.regex import Regex as BsonRegex
+except ImportError:
+    USE_BSON = False
+else:
+    USE_BSON = True
 
 
 from ivre import VERSION, config
@@ -196,6 +204,12 @@ NMAP_FINGERPRINT_IVRE_KEY = {
 logging.basicConfig()
 
 
+class InvalidIPAddress(ValueError):
+
+    def __init__(self, value: Any) -> None:
+        self.value = value
+
+
 def is_valid_ip(ipstr: AnyStr) -> bool:
     """Return True iff `ipstr` is a valid IP address."""
     if isinstance(ipstr, bytes):
@@ -229,13 +243,17 @@ def ip2int(ipstr: AnyStr) -> int:
     try:
         return cast(int, struct.unpack("!I", socket.inet_aton(data))[0])
     except socket.error:
-        val1: int
-        val2: int
+        pass
+    val1: int
+    val2: int
+    try:
         val1, val2 = struct.unpack(
             "!QQ",
             socket.inet_pton(socket.AF_INET6, data),
         )
-        return (val1 << 64) + val2
+    except socket.error as exc:
+        raise InvalidIPAddress(data) from exc
+    return (val1 << 64) + val2
 
 
 def force_ip2int(ipstr: Union[AnyStr, int]) -> int:
@@ -298,7 +316,7 @@ def ip2bin(ipval: Union[AnyStr, int]) -> bytes:
         try:
             data = ipval.decode()
         except UnicodeDecodeError as exc:
-            raise ValueError(f"Invalid IP address {ipval!r}") from exc
+            raise InvalidIPAddress(ipval) from exc
     else:
         data = ipval
     try:
@@ -310,7 +328,7 @@ def ip2bin(ipval: Union[AnyStr, int]) -> bytes:
     try:
         return socket.inet_pton(socket.AF_INET6, data)
     except socket.error as exc:
-        raise ValueError(f"Invalid IP address {ipval!r}") from exc
+        raise InvalidIPAddress(data) from exc
 
 
 def bin2ip(ipval: Union[AnyStr, int]) -> str:
@@ -328,7 +346,7 @@ def bin2ip(ipval: Union[AnyStr, int]) -> str:
             socket.inet_pton(socket.AF_INET6, ipval)
             return ipval
         except socket.error as exc:
-            raise ValueError(f"Invalid IP address {ipval!r}") from exc
+            raise InvalidIPAddress(ipval) from exc
     if isinstance(ipval, int):
         return int2ip(ipval)
     if ipval[:12] == b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff\xff":
@@ -602,7 +620,7 @@ def diff(doc1: Dict[str, Any], doc2: Dict[str, Any]) -> Dict[str, Any]:
         if key in ["categories"]:
             set1 = set(doc1[key])
             set2 = set(doc2[key])
-            res[key] = list(set1.symmetric_difference(set2))
+            res[key] = sorted(set1.symmetric_difference(set2))
             if not res[key]:
                 del res[key]
             continue
@@ -796,10 +814,10 @@ class FileOpener(BinaryIO):
     def writable(self) -> bool:
         return self.fdesc.writable()
 
-    def write(self, s: bytes) -> int:
+    def write(self, s: bytes) -> int:  # type: ignore[override]
         return self.fdesc.write(s)
 
-    def writelines(self, lines: Iterable[bytes]) -> None:
+    def writelines(self, lines: Iterable[bytes]) -> None:  # type: ignore[override]
         self.fdesc.writelines(lines)
 
 
@@ -807,18 +825,33 @@ def open_file(fname: str) -> FileOpener:
     return FileOpener(fname)
 
 
-def hash_file(fname: str, hashtype: str = "sha1") -> bytes:
-    """Compute a hash of data from a given file"""
-    with open_file(fname) as fdesc:
-        result = hashlib.new(hashtype)
-        for data in iter(lambda: fdesc.read(1048576), b""):
-            result.update(data)
-        return result.hexdigest().encode()
+def recursive_filelisting(
+    base_directories: Iterable[str], error: Optional[List[bool]] = None
+) -> Generator[str, None, None]:
+    """Iterator on filenames in base_directories. Ugly hack: error is a
+    one-element list that will be set to True if one of the directories in
+    base_directories does not exist.
+
+    """
+
+    for base_directory in base_directories:
+        if not os.path.exists(base_directory):
+            LOGGER.warning("directory %r does not exist", base_directory)
+            if error:
+                error[0] = True
+            continue
+        if not os.path.isdir(base_directory):
+            yield base_directory
+            continue
+        for root, _, files in os.walk(base_directory):
+            for leaffile in files:
+                yield os.path.join(root, leaffile)
 
 
 def serialize(obj: Any) -> str:
     """Return a JSON-compatible representation for `obj`"""
-    if isinstance(obj, REGEXP_T):
+    regexp_types = (REGEXP_T, BsonRegex) if USE_BSON else REGEXP_T
+    if isinstance(obj, regexp_types):  # type: ignore
         return "/%s/%s" % (
             obj.pattern,
             "".join(x.lower() for x in "ILMSXU" if getattr(re, x) & obj.flags),
@@ -1023,7 +1056,7 @@ def screenwords(imgdata: bytes) -> Optional[List[str]]:
         result = []
         size = MAXVALLEN
         for line in proc.stdout:
-            if size == 0:
+            if not size:
                 break
             for word_match in _WORDS.finditer(line):
                 word_bytes = word_match.group()
@@ -1096,6 +1129,10 @@ if USE_PIL:
 
         """
         img = PIL.Image.open(BytesIO(imgdata))
+        if img.mode == "RGBA":
+            new_img = PIL.Image.new("RGB", img.size, (255, 255, 255))
+            new_img.paste(img, mask=img.split()[3])
+            img = new_img
         bbox = _trim_image(img, tolerance)
         if bbox:
             newbbox = (
@@ -1182,7 +1219,7 @@ def guess_srv_port(port1: int, port2: int, proto: str = "tcp") -> int:
     ports = _PORTS.get(proto, {})
     val1, val2 = ports.get(port1, 0), ports.get(port2, 0)
     cmpval = (val1 > val2) - (val1 < val2)
-    if cmpval == 0:
+    if not cmpval:
         return (port2 > port1) - (port2 < port1)
     return cmpval
 
@@ -1216,11 +1253,21 @@ def _read_nmap_probes() -> None:
         elif line.startswith(b"Probe "):
             _NMAP_CUR_PROBE = []
             _NMAP_CUR_FALLBACK = []
+            probe: Optional[bytes]
             proto, name, probe = line[6:].split(b" ", 2)
-            if not (len(probe) >= 3 and probe[:2] == b"q|" and probe[-1:] == b"|"):
+            if not (len(probe) >= 3 and probe[:2] == b"q|"):
                 LOGGER.warning("Invalid nmap probe %r", probe)
-            else:
-                probe = nmap_decode_data(probe[2:-1].decode(), arbitrary_escapes=True)
+                return
+            while not probe[-1:] == b"|":
+                try:
+                    probe = probe.rsplit(b" ", 1)[-2]
+                except IndexError:
+                    LOGGER.warning("Invalid nmap probe %r", probe)
+                    probe = None
+                    break
+            if probe is None:
+                return
+            probe = nmap_decode_data(probe[2:-1].decode(), arbitrary_escapes=True)
             assert _NMAP_CUR_PROBE is not None
             assert _NMAP_CUR_FALLBACK is not None
             _NMAP_PROBES.setdefault(proto.lower().decode(), {})[name.decode()] = {
@@ -1397,7 +1444,7 @@ def _read_nmap_payloads() -> None:
     def _parse_line(line: str) -> Generator[str, None, None]:
         status = 0
         for c in line:
-            if status == 0:
+            if not status:
                 if c in {"#", "\r", "\n"}:
                     return
                 yield c
@@ -1442,6 +1489,10 @@ def _read_nmap_payloads() -> None:
                     cur_line.append(line)
         if cur_probe is not None:
             _NMAP_PAYLOADS[nmap_decode_data("".join(cur_line))] = cur_probe
+    except FileNotFoundError:
+        # This file has been removed from Nmap in 7.94 and is now
+        # obsolete.
+        pass
     except (AttributeError, AssertionError, TypeError, IOError):
         LOGGER.warning("Cannot read Nmap service fingerprint file.", exc_info=True)
     _NMAP_PAYLOADS_POPULATED = True
@@ -1493,9 +1544,9 @@ def find_ike_vendor_id(vendorid: bytes) -> Optional[bytes]:
     return None
 
 
-_WIRESHARK_MANUF_DB_LAST_ADDR: List[int] = []
-_WIRESHARK_MANUF_DB_VALUES: List[Optional[Tuple[str, Optional[str]]]] = []
-_WIRESHARK_MANUF_DB_POPULATED = False
+_MANUF_DB_LAST_ADDR: List[int] = []
+_MANUF_DB_VALUES: List[Optional[Tuple[str, Optional[str]]]] = []
+_MANUF_DB_POPULATED = False
 
 
 def _mac2int(value: str) -> int:
@@ -1514,9 +1565,11 @@ def _int2macmask(mask: int) -> int:
 
 
 def _read_wireshark_manuf_db() -> None:
-    global _WIRESHARK_MANUF_DB_POPULATED
+    global _MANUF_DB_POPULATED
 
-    if config.WIRESHARK_SHARE_PATH is None:
+    sep = re.compile("\t+")
+
+    if config.DATA_PATH is None:
         return
 
     def parse_line(line: str) -> None:
@@ -1528,10 +1581,10 @@ def _read_wireshark_manuf_db() -> None:
             return
         comment: Optional[str]
         try:
-            addr, manuf, comment = line.split("\t", 2)
+            addr, manuf, comment = sep.split(line, 2)
         except ValueError:
             try:
-                addr, manuf = line.split("\t", 1)
+                addr, manuf = sep.split(line, 1)
             except ValueError:
                 LOGGER.warning(
                     "Cannot parse a line from Wireshark manufacturer database [%r].",
@@ -1554,40 +1607,37 @@ def _read_wireshark_manuf_db() -> None:
                 line,
             )
             return
-        if (
-            _WIRESHARK_MANUF_DB_LAST_ADDR
-            and _WIRESHARK_MANUF_DB_LAST_ADDR[-1] != addr_int - 1
-        ):
-            _WIRESHARK_MANUF_DB_LAST_ADDR.append(addr_int - 1)
-            _WIRESHARK_MANUF_DB_VALUES.append(None)
-        elif _WIRESHARK_MANUF_DB_VALUES and _WIRESHARK_MANUF_DB_VALUES[-1] == (
+        if _MANUF_DB_LAST_ADDR and _MANUF_DB_LAST_ADDR[-1] != addr_int - 1:
+            _MANUF_DB_LAST_ADDR.append(addr_int - 1)
+            _MANUF_DB_VALUES.append(None)
+        elif _MANUF_DB_VALUES and _MANUF_DB_VALUES[-1] == (
             manuf,
             comment,
         ):
-            _WIRESHARK_MANUF_DB_LAST_ADDR.pop()
-            _WIRESHARK_MANUF_DB_VALUES.pop()
-        _WIRESHARK_MANUF_DB_LAST_ADDR.append(
+            _MANUF_DB_LAST_ADDR.pop()
+            _MANUF_DB_VALUES.pop()
+        _MANUF_DB_LAST_ADDR.append(
             (addr_int & _int2macmask(mask)) + 2 ** (48 - mask) - 1
         )
-        _WIRESHARK_MANUF_DB_VALUES.append((manuf, comment))
+        _MANUF_DB_VALUES.append((manuf, comment))
 
     try:
         with open(
-            os.path.join(config.WIRESHARK_SHARE_PATH, "manuf"), "r", encoding="utf8"
+            os.path.join(config.DATA_PATH, "manuf"), "r", encoding="utf8"
         ) as fdesc:
             for line in fdesc:
                 parse_line(line[:-1])
     except (AttributeError, TypeError, IOError):
         LOGGER.warning("Cannot read Wireshark manufacturer database.", exc_info=True)
-    _WIRESHARK_MANUF_DB_POPULATED = True
+    _MANUF_DB_POPULATED = True
 
 
-def get_wireshark_manuf_db() -> Tuple[
-    List[int], List[Optional[Tuple[str, Optional[str]]]]
-]:
-    if not _WIRESHARK_MANUF_DB_POPULATED:
+def get_wireshark_manuf_db() -> (
+    Tuple[List[int], List[Optional[Tuple[str, Optional[str]]]]]
+):
+    if not _MANUF_DB_POPULATED:
         _read_wireshark_manuf_db()
-    return _WIRESHARK_MANUF_DB_LAST_ADDR, _WIRESHARK_MANUF_DB_VALUES
+    return _MANUF_DB_LAST_ADDR, _MANUF_DB_VALUES
 
 
 def mac2manuf(mac: str) -> Optional[Tuple[str, Optional[str]]]:
@@ -1627,7 +1677,7 @@ def _nmap_decode_data(
     status = 0
     first_byte = -1
     for char in data:
-        if status == 0:
+        if not status:
             # not in an escape sequence
             if char == "\\":
                 status = 1
@@ -1846,7 +1896,15 @@ def parse_ssh_key(data: bytes) -> Dict[str, Any]:
             # convert integer to strings to prevent overflow errors
             # (e.g., "MongoDB can only handle up to 8-byte ints")
             for val in ["exponent", "modulus"]:
-                info[val] = str(info[val])
+                try:
+                    # with large SSH keys, it could lead to an exception
+                    info[val] = str(info[val])
+                except ValueError as exc:
+                    try:
+                        sys.set_int_max_str_digits(0)
+                    except AttributeError:
+                        raise exc from exc
+                    info[val] = str(info[val])
     elif keytype == "ssh-dss":
         info["bits"] = int(math.ceil(math.log(int(encode_hex(next(parsed)), 16), 2)))
     elif keytype == "ecdsa-sha2-nistp256":
@@ -2060,7 +2118,7 @@ PUBKEY_TYPES = {
     "dhpublicnumber": "dh",
 }
 
-PUBKEY_REV_TYPES = dict((val, key) for key, val in PUBKEY_TYPES.items())
+PUBKEY_REV_TYPES = {val: key for key, val in PUBKEY_TYPES.items()}
 
 
 def _parse_datetime(value: bytes) -> Optional[datetime.datetime]:
@@ -2099,7 +2157,11 @@ if USE_PYOPENSSL:
             hashtype: hashlib.new(hashtype, cert).hexdigest()
             for hashtype in ["md5", "sha1", "sha256"]
         }
-        data = osslc.load_certificate(osslc.FILETYPE_ASN1, cert)
+        try:
+            data = osslc.load_certificate(osslc.FILETYPE_ASN1, cert)
+        except osslc.Error:
+            LOGGER.warning("Cannot load certificate", exc_info=True)
+            return result
         result["subject_text"], result["subject"] = _parse_subject(data.get_subject())
         result["issuer_text"], result["issuer"] = _parse_subject(data.get_issuer())
         for i in range(data.get_extension_count()):
@@ -2139,7 +2201,14 @@ if USE_PYOPENSSL:
             # RSA
             numbers = pubkey.to_cryptography_key().public_numbers()
             result["pubkey"]["exponent"] = numbers.e
-            result["pubkey"]["modulus"] = str(numbers.n)
+            try:
+                result["pubkey"]["modulus"] = str(numbers.n)
+            except ValueError as exc:
+                try:
+                    sys.set_int_max_str_digits(0)
+                except AttributeError:
+                    raise exc from exc
+                result["pubkey"]["modulus"] = str(numbers.n)
         pubkey = pubkey.to_cryptography_key().public_bytes(
             Encoding.DER,
             PublicFormat.SubjectPublicKeyInfo,
@@ -2162,7 +2231,7 @@ else:
                     continue
                 curkey.append(char)
                 status += 1
-            elif status == 0:
+            elif not status:
                 # reading key
                 if char == " ":
                     status += 1
@@ -2259,9 +2328,9 @@ else:
                         for key, value in _parse_cert_subject(fdata_str)
                     ]
                     # replace '.' by '_' in keys to produce valid JSON
-                    result[field] = dict(
-                        (key.replace(".", "_"), value) for key, value in flddata
-                    )
+                    result[field] = {
+                        key.replace(".", "_"): value for key, value in flddata
+                    }
                     result["%s_text" % field] = "/".join(
                         "%s=%s" % item for item in flddata
                     )
@@ -2460,7 +2529,8 @@ def download_if_newer(
 
     """
     if processor is None:
-        processor = shutil.copyfileobj
+        processor = shutil.copyfileobj  # type: ignore[assignment]
+    assert processor is not None
     opener = build_opener()
     opener.addheaders = [("User-Agent", "IVRE/%s +https://ivre.rocks/" % VERSION)]
     try:
@@ -2476,20 +2546,29 @@ def download_if_newer(
                 ),
             )
         )
+    file_opened = False
     try:
-        with opener.open(url) as udesc, open(outfile, "wb") as wdesc:
-            processor(udesc, wdesc)
+        # attempt to remove the file only if it has been open()ed by
+        # us
+        with opener.open(url) as udesc:
+            # make sure opener.open(url) works before opening the file
+            with open(outfile, "wb") as wdesc:
+                file_opened = True
+                processor(udesc, wdesc)
     except HTTPError as exc:
         if exc.status == 304:
             LOGGER.debug("Won't download %s: our copy is up-to-date", url)
             return False
         LOGGER.error("Cannot download %s [%s]", url, exc)
+    except URLError as exc:
+        LOGGER.error("Cannot download %s [%s]", url, exc)
     except Exception as exc:
         LOGGER.error("Error processing %s [%s]", url, exc)
-        try:
-            os.unlink(outfile)
-        except FileNotFoundError:
-            pass
+        if file_opened:
+            try:
+                os.unlink(outfile)
+            except FileNotFoundError:
+                pass
         raise
     return True
 
@@ -2511,7 +2590,10 @@ def generic_processor(
     )
 
 
-def _fix_range(start: str, stop: str, label: str) -> Tuple[int, int, str]:
+T = TypeVar("T")
+
+
+def _fix_range(start: str, stop: str, label: T) -> Tuple[int, int, T]:
     return (
         ip2int(start) if ":" in start else ip2int(f"::ffff:{start}"),
         ip2int(stop) if ":" in stop else ip2int(f"::ffff:{stop}"),
@@ -2520,12 +2602,12 @@ def _fix_range(start: str, stop: str, label: str) -> Tuple[int, int, str]:
 
 
 def make_range_tables(
-    ranges: Iterable[Tuple[str, str, str]]
-) -> List[Tuple[int, Optional[str]]]:
-    ranges_sorted: List[Tuple[int, int, str]] = sorted(
+    ranges: Iterable[Tuple[str, str, T]]
+) -> List[Tuple[int, Optional[T]]]:
+    ranges_sorted: List[Tuple[int, int, T]] = sorted(
         (_fix_range(start, stop, label) for start, stop, label in ranges), reverse=True
     )
-    result: List[Tuple[int, Optional[str]]] = []
+    result: List[Tuple[int, Optional[T]]] = []
     prev = 0
     while ranges_sorted:
         start, stop, label = ranges_sorted.pop()
