@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 
 # This file is part of IVRE.
-# Copyright 2011 - 2023 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2024 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -22,40 +22,26 @@
 
 import struct
 from datetime import datetime
+from functools import reduce
 from textwrap import wrap
 
-try:
-    from importlib.metadata import entry_points
-except ImportError:
-    HAS_PLUGINS = False
-else:
-    HAS_PLUGINS = True
-
-
 from ivre import utils
 from ivre.active.cpe import add_cpe_values
 from ivre.active.data import (
     add_cert_hostnames,
     add_hostname,
     create_ssl_output,
-    set_auto_tags,
-    set_openports_attribute,
+    merge_host_docs,
 )
 from ivre.data import scanners
 from ivre.db import db
 from ivre.passive import SCHEMA_VERSION as PASSIVE_SCHEMA_VERSION
+from ivre.plugins import load_plugins
+from ivre.tags.active import set_auto_tags, set_openports_attribute
 from ivre.xmlnmap import SCHEMA_VERSION as ACTIVE_SCHEMA_VERSION
 from ivre.xmlnmap import add_service_hostname
 
-
-def load_plugins():
-    try:
-        my_entry_points = entry_points(group="ivre.plugins.view")
-    except TypeError:
-        my_entry_points = entry_points().get("ivre.plugins.view", [])
-    for entry_point in my_entry_points:
-        if entry_point.name.startswith("_install_"):
-            entry_point.load()(globals())
+MAX_RECORDS_IN_MEMORY = 16384
 
 
 def _extract_passive_HTTP_CLIENT_HEADER_SERVER(rec):
@@ -427,6 +413,8 @@ def _extract_passive_SSL_CLIENT(rec):
         return _extract_passive_SSL_cert(rec, cacert=True, server=False)
     if source == "ja3":
         return _extract_passive_SSL_CLIENT_ja3(rec)
+    if source == "ja4":
+        return _extract_passive_SSL_CLIENT_ja4(rec)
     return {}
 
 
@@ -563,6 +551,31 @@ def _extract_passive_SSL_CLIENT_ja3(rec):
     return {"ports": [port]}
 
 
+def _extract_passive_SSL_CLIENT_ja4(rec):
+    """Handle SSL client ja4 extraction."""
+    script = {"id": "ssl-ja4-client"}
+    script["output"] = rec["value"]
+    info = dict(rec.get("infos", {}), ja4=rec["value"])
+    script["ssl-ja4-client"] = [info]
+    port = {"port": -1, "scripts": [script]}
+    if rec["value"] in scanners.JA4_CLIENT_VALUES:
+        scanner, probe = scanners.JA4_CLIENT_VALUES[rec["value"]]
+        structured_output = {"scanners": [{"name": scanner}]}
+        if probe is not None:
+            structured_output["scanners"][0]["probes"] = [
+                {"proto": "tls", "name": probe}
+            ]
+        structured_output["probes"] = [{"proto": "tls", "value": rec["value"]}]
+        port["scripts"].append(
+            {
+                "id": "scanner",
+                "output": "Scanner:\n - %s [%s/tls]" % (scanner, rec["value"]),
+                "scanner": structured_output,
+            }
+        )
+    return {"ports": [port]}
+
+
 def _extract_passive_MAC_ADDRESS(rec):
     """Handle MAC addresses"""
     return {"addresses": {"mac": [rec["value"].lower()]}}
@@ -777,8 +790,6 @@ def passive_record_to_view(rec, category=None):
     if isinstance(function, dict):
         function = function.get(rec["source"], lambda _: {})
     outrec.update(function(rec))
-    set_auto_tags(outrec, update_openports=False)
-    set_openports_attribute(outrec)
     if category is not None:
         outrec["categories"] = [category]
     return outrec
@@ -815,44 +826,21 @@ def passive_to_view(flt, category=None):
             pass
 
 
-def from_passive(flt, category=None):
-    """Iterator over passive results, by address."""
-    records = passive_to_view(flt, category=category)
-    cur_addr = None
-    cur_rec = {}
-    for rec in records:
-        if cur_addr is None:
-            cur_addr = rec["addr"]
-            cur_rec = rec
-            continue
-        if cur_addr == rec["addr"]:
-            cur_rec = db.view.merge_host_docs(cur_rec, rec)
-            continue
-        # TODO: add_addr_info should be optional
-        cur_rec["infos"] = {}
-        for func in [db.data.country_byip, db.data.as_byip, db.data.location_byip]:
-            cur_rec["infos"].update(func(cur_addr) or {})
-        yield cur_rec
-        cur_rec = rec
-        cur_addr = rec["addr"]
-    if cur_rec:
-        yield cur_rec
-
-
 def nmap_record_to_view(rec, category=None):
     """Convert an nmap result in view."""
     if "_id" in rec:
         del rec["_id"]
-    if "scanid" in rec:
-        del rec["scanid"]
     if "source" in rec:
         if not rec["source"]:
             rec["source"] = []
-        elif not isinstance(rec["source"], list):
+        elif isinstance(rec["source"], list):
+            rec["source"] = sorted(set(rec["source"]))
+        else:
             rec["source"] = [rec["source"]]
-    rec.setdefault("categories", [])
+    categories = set(rec.get("categories", []))
     if category is not None:
-        rec["categories"].append(category)
+        categories.add(category)
+    rec["categories"] = sorted(categories)
     for port in rec.get("ports", []):
         if "screendata" in port:
             port["screendata"] = db.nmap.from_binary(port["screendata"])
@@ -862,11 +850,14 @@ def nmap_record_to_view(rec, category=None):
     return rec
 
 
-def from_nmap(flt, category=None):
-    """Return an Nmap entry in the View format."""
-    cur_addr = None
-    cur_rec = None
-    result = None
+def nmap_to_view(flt, category=None):
+    """Generates nmap entries in the View format.
+
+    Note that this entry is likely to have no sense in itself. This
+    function is intended to be used to format results for the merge
+    function.
+
+    """
     done = False
     skip = 0
     while not done:
@@ -884,51 +875,44 @@ def from_nmap(flt, category=None):
                 if "addr" not in rec:
                     skip += 1
                     continue
-                rec = nmap_record_to_view(rec, category=category)
-                if cur_addr is None:
-                    cur_addr = rec["addr"]
-                    cur_rec = rec
-                elif cur_addr == rec["addr"]:
-                    cur_rec = db.view.merge_host_docs(cur_rec, rec)
-                else:
-                    result = cur_rec
-                    cur_rec = rec
-                    cur_addr = rec["addr"]
-                    set_auto_tags(result)
-                    yield result
+                yield nmap_record_to_view(rec, category=category)
                 skip += 1
-            if cur_rec is not None:
-                set_auto_tags(cur_rec)
-                yield cur_rec
             done = True
         except db.nmap.cursor_timeout_exceptions:
             pass
 
 
-def to_view(itrs):
-    """Takes a list of iterators over view-formated results, and returns an
-    iterator over merged results, sorted by ip.
+def prepare_record(rec, datadb):
+    """Prepare a record before sending it to the view"""
+    for port in rec.get("ports", []):
+        if "screendata" in port:
+            port["screendata"] = db.view.to_binary(port["screendata"])
+        for script in port.get("scripts", []):
+            if "masscan" in script and "raw" in script["masscan"]:
+                script["masscan"]["raw"] = db.view.to_binary(script["masscan"]["raw"])
+    set_auto_tags(rec, update_openports=False)
+    set_openports_attribute(rec)
+    if datadb is not None:
+        rec.setdefault("infos", {})
+        addr = rec["addr"]
+        for func in [
+            datadb.country_byip,
+            datadb.as_byip,
+            datadb.location_byip,
+        ]:
+            rec["infos"].update(func(addr) or {})
+    return rec
+
+
+def to_view_parallel(itrs):
+    """Takes a list of iterators over view-formated results, and
+    returns an iterator over a list of results to be merged (by
+    parallel workers), sorted by ip.
 
     """
 
-    def next_record(rec, updt):
-        if rec is None:
-            return updt
-        return db.view.merge_host_docs(rec, updt)
-
     next_recs = []
 
-    def prepare_record(rec):
-        for port in rec.get("ports", []):
-            if "screendata" in port:
-                port["screendata"] = db.view.to_binary(port["screendata"])
-            for script in port.get("scripts", []):
-                if "masscan" in script and "raw" in script["masscan"]:
-                    script["masscan"]["raw"] = db.view.to_binary(
-                        script["masscan"]["raw"]
-                    )
-        return rec
-
     # We cannot use a `for itr in itrs` loop here because itrs is
     # modified in the loop.
     i = 0
@@ -943,7 +927,7 @@ def to_view(itrs):
         else:
             i += 1
     next_addrs = [rec["addr"] for rec in next_recs]
-    cur_rec = None
+    cur_recs = []
     cur_addr = min(next_addrs, key=utils.ip2int, default=None)
     while next_recs:
         # We cannot use a `for i in range(len(itrs))` loop because
@@ -951,7 +935,16 @@ def to_view(itrs):
         i = 0
         while i < len(itrs):
             while next_addrs[i] == cur_addr:
-                cur_rec = next_record(cur_rec, next_recs[i])
+                cur_recs.append(next_recs[i])
+                if len(cur_recs) >= MAX_RECORDS_IN_MEMORY:
+                    cur_recs = [
+                        reduce(
+                            lambda r1, r2: merge_host_docs(
+                                r1, r2, auto_tags=False, openports_attribute=False
+                            ),
+                            cur_recs,
+                        )
+                    ]
                 try:
                     next_recs[i] = next(itrs[i])
                 except StopIteration:
@@ -963,12 +956,60 @@ def to_view(itrs):
                 next_addrs[i] = next_recs[i]["addr"]
             i += 1
         if next_addrs and cur_addr not in next_addrs:
-            yield prepare_record(cur_rec)
-            cur_rec = None
+            yield cur_recs
+            cur_recs = []
             cur_addr = min(next_addrs, key=utils.ip2int)
-    if cur_rec is not None:
-        yield prepare_record(cur_rec)
+    if cur_recs:
+        yield cur_recs
 
 
-if HAS_PLUGINS:
-    load_plugins()
+def to_view(itrs, datadb):
+    """Takes a list of iterators over view-formated results, and returns an
+    iterator over merged results, sorted by ip.
+
+    """
+
+    next_recs = []
+
+    # We cannot use a `for itr in itrs` loop here because itrs is
+    # modified in the loop.
+    i = 0
+    while i < len(itrs):
+        try:
+            next_recs.append(next(itrs[i]))
+        except StopIteration:
+            # We need to remove the corresponding iterator from itrs,
+            # which happens to be the n-th where n is the current
+            # length of next_recs.
+            del itrs[len(next_recs)]  # Do not increment i here
+        else:
+            i += 1
+    next_addrs = [rec["addr"] for rec in next_recs]
+    cur_rec = {}
+    cur_addr = min(next_addrs, key=utils.ip2int, default=None)
+    while next_recs:
+        # We cannot use a `for i in range(len(itrs))` loop because
+        # itrs is modified in the loop.
+        i = 0
+        while i < len(itrs):
+            while next_addrs[i] == cur_addr:
+                cur_rec = merge_host_docs(cur_rec, next_recs[i])
+                try:
+                    next_recs[i] = next(itrs[i])
+                except StopIteration:
+                    del next_addrs[i]
+                    del next_recs[i]
+                    del itrs[i]
+                    i -= 1  # Do not increment i here
+                    break
+                next_addrs[i] = next_recs[i]["addr"]
+            i += 1
+        if next_addrs and cur_addr not in next_addrs:
+            yield prepare_record(cur_rec, datadb)
+            cur_rec = {}
+            cur_addr = min(next_addrs, key=utils.ip2int)
+    if cur_rec:
+        yield prepare_record(cur_rec, datadb)
+
+
+load_plugins("ivre.plugins.view", globals())
