@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 
 # This file is part of IVRE.
-# Copyright 2011 - 2022 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2024 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@ from bottle import Bottle, abort, request, response
 
 from ivre import VERSION, config, utils
 from ivre.db import db
+from ivre.tags.active import set_auto_tags, set_openports_attribute
 from ivre.view import nmap_record_to_view
 from ivre.web import utils as webutils
 
@@ -69,7 +70,6 @@ def check_referer(func):
 
     @wraps(func)
     def _newfunc(*args, **kargs):
-
         # Header with an existing X-API-Key header or an
         # Authorization: Bearer XXX are OK. Note: IVRE does not check
         # those values, they only serve as anti-CSRF protections.
@@ -630,11 +630,10 @@ def get_nmap(subdb):
     # XXX-WORKAROUND-PGSQL
     # for rec in result:
     for i, rec in enumerate(result):
-        for fld in ["_id", "scanid"]:
-            try:
-                del rec[fld]
-            except KeyError:
-                pass
+        try:
+            del rec["_id"]
+        except KeyError:
+            pass
         if flt_params.ipsasnumbers:
             rec["addr"] = utils.force_ip2int(rec["addr"])
         if not flt_params.datesasstrings:
@@ -669,7 +668,7 @@ def get_nmap(subdb):
             yield "%s\n" % json.dumps(rec, default=utils.serialize)
         else:
             yield "%s\t%s" % (
-                "" if i == 0 else ",\n",
+                ",\n" if i else "",
                 json.dumps(rec, default=utils.serialize),
             )
         check = subdb.cmp_schema_version_host(rec)
@@ -738,15 +737,23 @@ def parse_form():
 
 def import_files(subdb, source, categories, files):
     count = 0
-    categories = list(categories)
+    categories = sorted(categories)
     if subdb == "view":
 
         def callback(x):
-            db.view.start_store_hosts()
-            res = db.view.store_or_merge_host(nmap_record_to_view(x))
-            db.view.stop_store_hosts()
-            return res
+            result = nmap_record_to_view(x)
+            set_auto_tags(result, update_openports=False)
+            set_openports_attribute(result)
+            result["infos"] = {}
+            for func in [
+                db.data.country_byip,
+                db.data.as_byip,
+                db.data.location_byip,
+            ]:
+                result["infos"].update(func(result["addr"]) or {})
+            db.view.store_or_merge_host(result)
 
+        db.view.start_store_hosts()
     else:
         callback = None
     for fileelt in files:
@@ -762,6 +769,8 @@ def import_files(subdb, source, categories, files):
                 utils.LOGGER.warning("Could not import %s", fdesc.name)
         except Exception:
             utils.LOGGER.warning("Could not import %s", fdesc.name, exc_info=True)
+    if callback is not None:
+        db.view.stop_store_hosts()
     return count
 
 
@@ -1044,7 +1053,7 @@ def get_passive():
             yield "%s\n" % json.dumps(rec, default=utils.serialize)
         else:
             yield "%s\t%s" % (
-                "" if i == 0 else ",\n",
+                ",\n" if i else "",
                 json.dumps(rec, default=utils.serialize),
             )
         if flt_params.limit and i + 1 >= flt_params.limit:
