@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of IVRE.
-# Copyright 2011 - 2022 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2024 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -261,7 +261,7 @@ def flt_from_query(dbase, query, base_flt=None):
             )
         )
 
-    for (neg, param, value) in query:
+    for neg, param, value in query:
         if not neg and param == "skip":
             skip = int(value)
         elif not neg and param == "limit":
@@ -302,15 +302,15 @@ def flt_from_query(dbase, query, base_flt=None):
             flt = dbase.flt_and(
                 flt, dbase.searchcategory(utils.str2regexp(value), neg=neg)
             )
-        elif param == "country":
+        elif param == "country" and hasattr(dbase, "searchcountry"):
             flt = dbase.flt_and(
                 flt, dbase.searchcountry(utils.str2list(value.upper()), neg=neg)
             )
-        elif param == "city":
+        elif param == "city" and hasattr(dbase, "searchcity"):
             flt = dbase.flt_and(flt, dbase.searchcity(utils.str2regexp(value), neg=neg))
-        elif param == "asnum":
+        elif param == "asnum" and hasattr(dbase, "searchasnum"):
             flt = dbase.flt_and(flt, dbase.searchasnum(utils.str2list(value), neg=neg))
-        elif param == "asname":
+        elif param == "asname" and hasattr(dbase, "searchasname"):
             flt = dbase.flt_and(
                 flt, dbase.searchasname(utils.str2regexp(value), neg=neg)
             )
@@ -602,6 +602,14 @@ def flt_from_query(dbase, query, base_flt=None):
                     neg=neg,
                 ),
             )
+        elif param == "ssl-ja4-client":
+            flt = dbase.flt_and(
+                flt,
+                dbase.searchja4client(
+                    value=(None if value is None else utils.str2regexp(value)),
+                    neg=neg,
+                ),
+            )
         elif param == "ssl-ja3-server":
             if value is None:
                 # There are no additional arguments
@@ -708,9 +716,11 @@ def flt_from_query(dbase, query, base_flt=None):
             for proto, ports in protos.items():
                 flt = dbase.flt_and(
                     flt,
-                    dbase.searchport(ports[0], protocol=proto, state=param)
-                    if len(ports) == 1
-                    else dbase.searchports(ports, protocol=proto, state=param),
+                    (
+                        dbase.searchport(ports[0], protocol=proto, state=param)
+                        if len(ports) == 1
+                        else dbase.searchports(ports, protocol=proto, state=param)
+                    ),
                 )
         elif not neg and param == "otheropenport":
             flt = dbase.flt_and(
@@ -776,7 +786,7 @@ def flt_from_query(dbase, query, base_flt=None):
                 flt = dbase.flt_and(flt, dbase.searchcpe(**cpe_kwargs))
             else:
                 flt = dbase.flt_and(flt, dbase.searchcpe())
-        elif param == "tag":
+        elif param == "tag" and hasattr(dbase, "searchtag"):
             if value:
                 if ":" in value:
                     tag_val, tag_info = value.split(":", 1)
