@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 
 # This file is part of IVRE.
-# Copyright 2011 - 2022 Pierre LALET <pierre@droids-corp.org>
+# Copyright 2011 - 2023 Pierre LALET <pierre@droids-corp.org>
 #
 # IVRE is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by
@@ -246,10 +246,11 @@ def zgrap_parser_http(
                         "" if version is None else (", version %s" % version),
                     ),
                     "http-app": [
-                        dict(
-                            {"path": path, "application": "Centreon"},
+                        {
+                            "path": path,
+                            "application": "Centreon",
                             **({} if version is None else {"version": version}),
-                        )
+                        }
                     ],
                 }
             )
